import axios from 'axios';
import qs from '../res/qs';
import {layer} from './layer';
import string_tool from './string_tool';


/* @flow weak */

let total = 0;

const urlErrorMsg = '接口地址必须为非空字符';
const notJsonMsg = '服务端未返回有效json数据';
const alertTitle = '出错啦';
const serverError = '服务器未响应，或接口地址有误';
const contentType = {
  get: undefined,
  post: 'application/x-www-form-urlencoded; charset=UTF-8',
  postJson: 'application/json; charset=UTF-8',
  file: 'multipart/form-data'
};

/**
 * 启动了一个Ajax请求
 * 如果之前未显示加载动画，则显示加载动画
 */
function onStart() {
  if (total === 0) {
    layer.loading();
  }
  total++;
}

/**
 * Ajax请求结束
 * 当进行中的请求数为0时，关闭加载动画
 */
function onEnd(result) {
  total--;
  if (total <= 0) {
    layer.hideLoading();
  }
  return result;
}


axios.interceptors.request.use(config => {
  return config
}, error => {
  return Promise.reject(error)
});


/*
axios.interceptors.response.use(response => {
  return response
}, error => {
  return Promise.resolve(error.response)
});
*/


class BaseAjax {

  constructor() {
    this.pageMode = 'limit';      //分页模式
    this.onlyData = false;        //在自动处理模式下，是否仅返回data，以避免多余嵌套。默认为false。可在main.js中设置该值为true
    this.OriginalData = undefined;  //接口返回的原始数据。用于在自动处理模式下需要访问原json的场景。仅保留最后一次的Ajax请求结果，所以在多个请求时无法正常使用。目前暂无应用场景
    this.header = {};             //	该对象下的所有数据均会添加到Ajax请求头中。主要用于发送token
    this.slient = false;          //是否静默请求
  }

  newHeader(type) {
    const data = {
      'X-Requested-With': 'XMLHttpRequest',
      //'timeStamp': new Date().valueOf()
      'Content-Type': contentType[type],
    };
    Object.assign(data, this.header);
    return data;
  }

  createData(url, params = {}, type = 'get') {
    let postData = {
      url,
      timeout: 30000,
      headers: this.newHeader(type),
      method: type
    };
    switch (type) {
      case 'get':
        postData.params = params;
        break;
      case 'post':
        postData.data = qs.stringify(params);
        break;
      case 'postJson':
        postData.data = params;
        postData.method = 'post';
        break;
    }
    return postData;
  }

  /**
   * post方式提交值对型数据
   * @param url
   * @param params
   * @param autoFilter - 自动处理结果
   * @returns {Promise<AxiosResponse<any>>}
   */
  post(url: string, params: any, autoFilter: boolean = true): Promise<any> {
    if (typeof url !== 'string' || url.length === 0) {
      return Promise.reject(urlErrorMsg);
    }
    this.onStart('post form', url, params);

    const result = axios(this.createData(url, params, 'post')).catch((err) => {
      return onServerError(err, url, params);
    }).then(result => {
      this.onEnd(result, url, params);
      return result;
    });
    if (autoFilter) {
      return result.then(res => {
        return checkStatus(res, url, params);
      }).then((res) => {
        return checkSuccess(res, url, params);
      })
    } else {
      return result;
    }
  }


  /**
   * post方式提交值对型数据
   * @param url
   * @param params
   * @param autoFilter - 自动处理结果
   * @returns {Promise<AxiosResponse<any>>}
   */
  getBlob(url: string, params: any): Promise<any> {
    if (typeof url !== 'string' || url.length === 0) {
      return Promise.reject(urlErrorMsg);
    }
    this.onStart('get blob', url, params);

    return axios({
      method: 'get',
      url,
      data: qs.stringify(params),
      timeout: 30000,
      responseType: 'blob',
      headers: this.newHeader()

    }).catch((err) => {
      return onServerError(err, url, params);
    }).then(result => {
      this.onEnd(result, url, params);
      return result.data;
    });
  }


  /**
   * 上传文件
   * @param url
   * @param params
   * @param autoFilter - 自动处理结果
   * @returns {Promise<AxiosResponse<any>>}
   */
  upload(url: string, params: any, autoFilter: boolean = true, onUploadProgress): Promise<any> {
    if (typeof url !== 'string' || url.length === 0) {
      return Promise.reject(urlErrorMsg);
    }
    this.onStart('upload', url, params);

    const result = axios({
      method: 'post',
      url,
      data: params,
      timeout: 1800000,
      headers: this.newHeader('file'),
      onUploadProgress: function (progressEvent) {
        if (progressEvent.lengthComputable) {
          onUploadProgress(parseInt((progressEvent.loaded / progressEvent.total) * 100));
        }
      }
    }).catch((err) => {
      return onServerError(err, url, params);
    }).then(result => {
      this.onEnd(result, url, params);
      return result;
    });
    if (autoFilter) {
      return result.then(res => {
        return checkStatus(res, url, params);
      }).then((res) => {
        return checkSuccess(res, url, params);
      })
    } else {
      return result;
    }
  }

  /**
   *
   * @param url
   * @param params
   * @param autoFilter
   * @returns {*}
   */
  postJson(url: string, params: any, autoFilter: boolean = true): Promise<any> {
    if (typeof url !== 'string' || url.length === 0) {
      return Promise.reject(urlErrorMsg);
    }
    this.onStart('post json', url, params);
    const result = axios(this.createData(url, params, 'postJson')).catch((err) => {
      return onServerError(err, url, params);
    }).then(result => {
      this.onEnd(result, url, params);
      return result;
    });
    if (autoFilter) {
      return result.then(res => {
        return checkStatus(res, url, params);
      }).then((res) => {
        return checkSuccess(res, url, params);
      })
    } else {
      return result;
    }
  }

  /**
   * get方式提交数据
   * @param url
   * @param params
   * @param autoFilter - 自动处理结果
   * @returns {Promise<AxiosResponse<any>>}
   */
  get(url: string, params: any, autoFilter: boolean = true): Promise<any> {
    if (typeof url !== 'string' || url.length === 0) {
      return Promise.reject(urlErrorMsg);
    }
    this.onStart('get', url, params);
    const result = axios(this.createData(url, params, 'get')).catch((err) => {
      return onServerError(err, url, params);
    }).then(result => {
      this.onEnd(result, url, params);
      return result;
    });
    if (autoFilter) {
      return result.then(res => {
        return checkStatus(res, url, params);
      }).then((res) => {
        return checkSuccess(res, url, params);
      });
      /*
      .then(res => {
      if (res && res.data) {
        if (Array.isArray(res.data.rows)) {
          res.total = res.data.total;
          res.data = res.rows;
        }
      }
      return res;});
      */

    } else {
      return result;
    }
  }

  /**
   * 静默请求
   * @param url
   * @param params
   * @param type
   * @returns {Promise<AxiosResponse<any> | never | never>}
   */
  back(url, params = {}, type = 'get') {
    let postData = {
      url,
      timeout: 30000,
      headers: this.newHeader(type),
      method: type
    };
    switch (type) {
      case 'get':
        postData.params = params;
        break;
      case 'post':
        postData.data = qs.stringify(params);
        break;
      case 'postJson':
        postData.data = params;
        postData.method = 'post';
        break;
    }
    this.onStart('静默请求' + type, url, params);
    return axios(postData).catch(err => {
      this.onError(err, url, params);
    }).then(result => {
      this.onEnd(result, url, params);
      if (this.onlyData) {
        return result.data.data;
      } else {

        return result.data;
      }
    })
  }


  /**
   * get方式获取非Json数据
   * @param url
   * @param params
   * @returns {Promise<AxiosResponse<any>>}
   */
  getText(url: string, params: any): Promise<any> {
    if (typeof url !== 'string' || url.length === 0) {
      return Promise.reject(urlErrorMsg);
    }
    this.onStart('get 文本', url, params);
    return axios({
      method: 'get',
      url,
      params,
      timeout: 30000,
      headers: this.newHeader()
    }).catch((err) => {
      return onServerError(err, url, params);
    }).then(result => {
      this.onEnd(result, url, params);
      return result.data;
    })
  }

  /**
   * get方式加载分页数据
   * @param {string} url
   * @param {number} page - 当前页码
   * @param {object} opt - 回调
   * @param {string} mode - 接口调用方式
   */
  getPage(url: string, page: number, opt: any, mode = 'get'): Promise<any> {
    if (typeof url !== 'string' || url.length === 0) {
      return Promise.reject(urlErrorMsg);
    }
    opt = Object.assign({
      paras: {},
      limit: 20,
      onEnd: function (isEmpty) {
      },
      filter: function (result) {
        return result;
      }
    }, opt || {});
    const paras = opt.paras;
    switch (this.pageMode) {
      case 'page':
        paras.pageSize = opt.limit;
        paras.page = page;
        break;
      default:
        //limit模式
        paras.offset = (page - 1) * opt.limit;
        paras.limit = opt.limit;
        break;
    }

    return this[mode](url, paras).then(function (result) {
      if (!result) {
        opt.onEnd(page === 1);
        return {
          total: 0,
          data: []
        }
      }
      result = opt.filter(result);
      result.total = parseInt(result.total);//后台给出的可能是字符串
      result.data = result.data || [];
      if (result.data.length === 0) {
        opt.onEnd(page === 1);
      } else {
        // 如果当前页数据未满，则视为已到末端
        if (result.total <= (opt.limit * page)) {
          opt.onEnd(false);
        }
      }
      return result;
    });
  }

  onStart(type, url, paras) {
    if (process.env.NODE_ENV === 'development') {
      console.group('Ajax请求开始', url);
      console.log('请求方式', type);
      if (paras) {
        console.log('发送数据', paras);
      }
      console.groupEnd();
    }
    if (!this.slient) {
      onStart();
    }
  }


  onEnd(result, url, paras) {
    if (process.env.NODE_ENV === 'development') {
      console.group('Ajax请求结束', url);
      if (paras) {
        console.log('发送数据', paras);
      }
      console.log('请求结果', result);
      console.groupEnd();
    }
    if (!this.slient) {
      onEnd(result);
    } else {//静默模式 取消静默
      this.slient = false;
    }
  }

  /**
   * 属于预处理函数
   *
   * @param res
   * @returns {*}
   */
  filter(res) {
    return res;
  }

  /**
   * 错误处理
   * @param code
   */
  onError(code, msg, url, paras) {
    console.group('服务端返回错误信息');
    console.error(code, msg);
    console.log(url);
    if (paras) {
      console.log(paras);
    }
    console.groupEnd();
  }

  /**
   * GET方式请求数据
   * 不显示动画，不自动处理数据
   * @param url
   * @param params
   * @returns {Promise<AxiosResponse<any> | never>}
   * @constructor
   */
  GET(url: string, params: any): Promise<any> {
    return axios(this.createData(url, params, 'get')).then(toData);
  }

  POST(url: string, params: any): Promise<any> {
    return axios(this.createData(url, params, 'post')).then(toData);
  }

  POST_JSON(url: string, params: any): Promise<any> {
    return axios(this.createData(url, params, 'postJson')).then(toData);
  }
}

const Ajax = new BaseAjax();

/**
 * 验证返回状态，以及是否返回json格式数据
 * @param response
 * @returns {Promise<never>}
 */
function checkStatus(response, url, paras) {
  if (response === undefined) {
    layer.alert('可能您正在进行跨域请求，或其它原因', alertTitle, 'net');
    Ajax.onError(response.status, '跨域请求，或网络问题', url, paras);
    return Promise.reject('网络错误');
  }
  if (response.status === 200 || response.status === 304) {
    if (typeof response.data === 'object') {
      return response.data;
    } else {
      layer.alert(notJsonMsg, alertTitle, 'json');
      Ajax.onError(response.status, '服务断未返回有效JSON数据', url, paras);
      return Promise.reject(notJsonMsg);
    }
  } else {
    layer.alert(response.statusText, alertTitle, 'status');
    Ajax.onError(response.status, '未定义错误类型', url, paras);
    return Promise.reject(response.statusText);
  }
}

function toData(response) {
  if (response === undefined) {
    return Promise.reject('网络错误');
  }
  if (response.status === 200 || response.status === 304) {
    if (typeof response.data === 'object') {
      return response.data;
    } else {
      return Promise.reject('服务端未返回json');
    }
  } else {
    let msg = response.status + '错误';
    switch (response.status) {
      case 400:
        msg = '400:提交的数据不符';
        break;
      case 404:
        msg = '404:接口路径错误';
        break;
      case 500:
        msg = '500:服务器内部错误';
        break;
    }
    return Promise.reject(msg);
  }

}

/**
 * 验证Json结果中success是否为true
 * @param result
 * @returns {*}
 */
function checkSuccess(result, url, paras) {
  if (result.success || result.code === 0) {
    //如果启用了仅返回data，则把原始数据保存到OriginalData属性
    if (Ajax.onlyData) {
      Ajax.OriginalData = result;
      return result.data;
    } else {
      return result;
    }
  }
  //如果onError未阻止弹窗，则自动弹出错误信息
  //该事件需要在应用中处理
  if (!Ajax.onError(result.code, result.msg, url, paras)) {
    layer.alert(string_tool.maxLen(result.msg, 200), alertTitle, 'custom');
  }
  return Promise.reject(result.msg);
}

/**
 * 服务器未响应时处理
 */
function onServerError(error, url, params) {
  console.group('Ajax请求失败', url);
  if (params) {
    console.log(params);
  }
  console.error(error);
  console.groupEnd();
  onEnd();
  Ajax.onError(400, error, url, params);
  return Promise.reject(serverError);
}
export default Ajax;

export {contentType, Ajax}
