import {getType, } from './basic';
import {layer} from './layer';
import {Ajax} from './ajax';

const Option = {
  url: '',    // 接口地址
  ratio: 0.6,  // 图片压缩比 0-1之间的数字
  max: 0,    // 可上传的文件数量，小于1为无限
  min: 0,
  auto: false,
  allowTypes: ['img']
};

export class File {
  constructor() {
    this.file = {};
  }
}

class ImgUpload {
  constructor(opt) {
    this.opt = Object.assign({}, Option, opt);
    this.data = [];
  }


  /**
   * 图片压缩
   * @param img - img对象
   * @param {number} prop - 压缩比，默认0.6
   * @returns {string} - 二进制数据
   */
  static imgCompress(img, prop = 0.6) {
    const canvas = document.createElement('canvas');
    const ctx = canvas.getContext('2d');
    const tCanvas = document.createElement('canvas');
    const tctx = tCanvas.getContext('2d');

    const initSize = img.src.length;
    let width = img.width;
    let height = img.height;
    // 如果图片大于四百万像素，计算压缩比并将大小压至400万以下
    let ratio;
    if ((ratio = width * height / 4000000) > 1) {
      ratio = Math.sqrt(ratio);
      width /= ratio;
      height /= ratio;
    } else {
      ratio = 1;
    }
    canvas.width = width;
    canvas.height = height;
    // 铺底色
    ctx.fillStyle = '#fff';
    ctx.fillRect(0, 0, canvas.width, canvas.height);
    // 如果图片像素大于100万则使用瓦片绘制
    let count;
    if ((count = width * height / 1000000) > 1) {
      count = ~~(Math.sqrt(count) + 1); // 计算要分成多少块瓦片
      // 计算每块瓦片的宽和高
      const nw = ~~(width / count);
      const nh = ~~(height / count);
      tCanvas.width = nw;
      tCanvas.height = nh;
      for (let i = 0; i < count; i++) {
        for (let j = 0; j < count; j++) {
          tctx.drawImage(img, i * nw * ratio, j * nh * ratio, nw * ratio, nh * ratio, 0, 0, nw, nh);
          ctx.drawImage(tCanvas, i * nw, j * nh, nw, nh);
        }
      }
    } else {
      ctx.drawImage(img, 0, 0, width, height);
    }
    // 进行压缩
    const ndata = canvas.toDataURL('image/jpeg', prop);
    console.log('压缩前：' + initSize);
    console.log('压缩后：' + ndata.length);
    console.log('压缩率：' + ~~(100 * (initSize - ndata.length) / initSize) + '%');
    tCanvas.width = tCanvas.height = canvas.width = canvas.height = 0;
    return ndata;
  }

  /**
   * 转化文档类型
   * @param {string} type
   * @returns {string}
   * @constructor
   */
  static GetFileType(type) {
    console.log(type);
    switch (type) {
      case 'image/png':
      case 'image/jpeg':
      case 'image/gif':
        return 'img';
      case 'audio/mp3':
        return 'music';
      default:
        return 'other';
    }
  }

  /**
   *
   * @param file
   */
  addFile(file) {
    const opt = this.opt, that = this;
    const trueType = ImgUpload.GetFileType(file.type);
    return new Promise(function (resolve, reject) {
      const reader = new FileReader();
      reader.onload = () => {
        if (opt.allowTypes.indexOf(trueType) > -1) {
          that.data.push({
            file: file,
            src: reader.result,
            fileType: file.type,
            type: trueType,
            name: file.name,
            upload: false
          });
          resolve(reader.result);
        } else {
          reject();
          this.onError('不允许上传该类型的文档');
        }
      };
      reader.readAsDataURL(file);
    });
  }

  /**
   *
   * @param file
   */
  addFiles(files) {
    const len = files.length;
    const opt = this.opt;
    if (opt.max > 0) {
      if (len + this.data.length > opt.max) {
        this.onError('当前最多允许上传' + opt.max + '张照片');
        return;
      }
    }
    for (let i = 0; i < len; i++) {
      this.addFile(files[i]);
    }
  }

  /**
   * 添加文件
   * 由file类型的input的change触发
   * @param event
   */
  add(event) {
    const files = event.target.files;
    if (files.length === 0) {
      console.log('未选择任何文件');
      return;
    }

    if (this.mayAdd(files.length)) {
      for (const file of files) {
        this.addFile(file);
      }
    }
  }

  /**
   * 导入已上传的文件 用于编辑模式
   * @param {array|string} value
   */
  loadFiles(value) {
    switch (getType(value)) {
      case 'String':
        if (value) {
          const arr = value.split(',');
          arr.forEach((item) => {
            this.data.push({
              dataid: 0,
              src: item,
              type: '',
              name: item,
              upload: true
            });
          });
        }
        break;
      case 'Array':
        value.forEach((item) => {
          this.data.push({
            dataid: item.dataid || 0,
            src: item.src,
            type: item.type || '',
            name: item.name || item.src,
            upload: true
          });
        });
        break;
    }
  }

  clear() {
    this.data.length = 0;
  }

  /**
   * 获取未上传的文件数量
   * @returns {number}
   */
  get waitUpload() {
    return this.data.filter(function (item) {
      return !item.upload;
    }).length;
  }

  onError(msg) {
    layer.alert(msg);
  }

  /**
   * 任务完成后执行
   */
  onFinish() {
  }

  /**
   * 显示上传进度
   * @param {number} index - 当前上传的文件序号，从1开始
   * @param {number} max - 需要上传的文件总数
   * @param {number} prop - 当前上传的文件上传进度0-100
   */
  onProgress(index, max, prop) {

  }

  /**
   * 单个文件上传后执行
   * @param json
   * @param {File} file
   */
  onUploadSuccess(json, file) {
    file.src = json.url;
  }

  upload() {
    const task = this.data.filter(function (item) {
      return !item.upload;
    });
    if (task.length === 0) {
      this.onFinish();
      return;
    }
    let index = 0;
    const len = task.length, that = this, opt = this.opt;
    submit();

    function submit() {
      const file = task[index];
      index++;
      // 图片类型 并启用压缩
      if (file.type === 'img' && that.opt.ratio < 1) {
        const img = new Image();
        img.src = file.src;
        const data = ImgUpload.imgCompress(img, opt.ratio);
        send(data, file);
      } else {
        send(file.src, file);
      }
    }

    /**
     * 上传图片
     * @param data
     */
    function send(data, file) {
      const text = window.atob(data.split(',')[1]);
      const buffer = new Uint8Array(text.length);
      for (let i = 0; i < text.length; i++) {
        buffer[i] = text.charCodeAt(i);
      }
      const blob = new Blob([buffer], {
        type: file.fileType
      });
      const xhr = new XMLHttpRequest();
      const formdata = new FormData();
      formdata.append('file', blob, file.name);
      xhr.open('post', opt.url);
      xhr.setRequestHeader('token', Ajax.header.token);
      xhr.onreadystatechange = function () {
        if (xhr.readyState === 4 && xhr.status === 200) {
          file.upload = true; // 将文件标记为已上传
          that.onUploadSuccess(JSON.parse(xhr.responseText), file);
          if (index < len) {
            submit();
          } else {
            that.onFinish();
          }
        }
      };
      xhr.upload.addEventListener('progress', function (e) {
        that.onProgress(index, len, ~~(100 * e.loaded / e.total) / 2);
      }, false);
      xhr.send(formdata);

    }

  }

  /**
   * 判断可否添加文件
   * @param {number} num - 新选择的文件数量
   */
  mayAdd(num) {
    if (this.opt.max < 1) {
      return true;
    }
    if (this.data.length + num > this.opt.max) {
      this.onError('所选文件超出数量限制');
      return false;
    } else {
      return true;
    }
  }

  get result() {
    const data = [];
    this.data.forEach(file => {
      if (typeof file.src === 'string') {
        data.push(file.src);
      }
    });
    return data.toString();
  }

  remove(file) {
    this.data.splice(this.data.indexOf(file), 1);
  }
}

export default ImgUpload;
